@@ -14,7 +14,7 @@ from pyramid_hypernova.types import Job
 from pyramid_hypernova.types import JobResult
 
 
-def create_fallback_response(jobs, throw_client_error, json_encoder, error=None):
+def create_fallback_response(jobs, throw_client_error, json_encoder, error=None, display_error_stack=False):
     """Create a response dict for falling back to client-side rendering.
 
     :rtype: Dict[str, Job]
@@ -22,7 +22,13 @@ def create_fallback_response(jobs, throw_client_error, json_encoder, error=None)
     return {
         identifier: JobResult(
             error=error,
-            html=render_blank_markup(identifier, job, throw_client_error, json_encoder),
+            html=render_blank_markup(
+                identifier,
+                job,
+                throw_client_error,
+                json_encoder,
+                error=error if display_error_stack else None,
+            ),
             job=job,
         )
         for identifier, job in jobs.items()
@@ -52,7 +58,8 @@ class BatchRequest:
         plugin_controller,
         pyramid_request,
         max_batch_size=None,
-        json_encoder=JSONEncoder()
+        json_encoder=JSONEncoder(),
+        display_error_stack=False
     ):
         self.get_job_group_url = get_job_group_url
         self.jobs = {}
@@ -60,6 +67,7 @@ class BatchRequest:
         self.max_batch_size = max_batch_size
         self.json_encoder = json_encoder
         self.pyramid_request = pyramid_request
+        self.display_error_stack = display_error_stack
 
     def render(self, name, data, context=None):
         if context is None:  # pragma: no cover
@@ -119,7 +127,10 @@ class BatchRequest:
                     message=response_json['error']['message'],
                     stack=response_json['error']['stack'],
                 )
-                pyramid_response = create_fallback_response(jobs, True, self.json_encoder, error)
+
+                pyramid_response = create_fallback_response(
+                    jobs, True, self.json_encoder, error, self.display_error_stack
+                )
                 self.plugin_controller.on_error(error, jobs, self.pyramid_request)
             else:
                 pyramid_response = self._parse_response(response_json)
@@ -128,14 +139,22 @@ class BatchRequest:
         except (HypernovaQueryError, ValueError) as e:
             # the service is unhealthy. fall back to client-side rendering
             __, __, exc_traceback = sys.exc_info()
-
-            error = HypernovaError(
-                type(e).__name__,
-                str(e),
-                [line.rstrip('\n') for line in traceback.format_tb(exc_traceback)],
-            )
+            # We allow clients to send specific error data with their response that they may want to surface.
+            # Check if any has been propagated, otherwise proceed with generic HypernovaError
+            if e.error_data:
+                error = HypernovaError(
+                    name=e.error_data.name,
+                    message=e.error_data.message,
+                    stack=e.error_data.stack,
+                )
+            else:
+                error = HypernovaError(
+                    type(e).__name__,
+                    str(e),
+                    [line.rstrip('\n') for line in traceback.format_tb(exc_traceback)],
+                )
             self.plugin_controller.on_error(error, jobs, self.pyramid_request)
-            pyramid_response = create_fallback_response(jobs, True, self.json_encoder, error)
+            pyramid_response = create_fallback_response(jobs, True, self.json_encoder, error, self.display_error_stack)
 
         return pyramid_response
 
@@ -145,7 +164,6 @@ class BatchRequest:
         :rtype: Dict[str, JobResult]
         """
         self.jobs = self.plugin_controller.prepare_request(self.jobs, self.pyramid_request)
-
         response = {}
 
         if self.jobs and self.plugin_controller.should_send_request(self.jobs, self.pyramid_request):
